package pt.unl.fct.di.novasys.babel.protocols.secure.dissemination.messages;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.SignatureException;
import java.security.UnrecoverableEntryException;
import java.security.cert.X509Certificate;
import java.util.UUID;

import pt.unl.fct.di.novasys.babel.core.BabelSecurity;
import pt.unl.fct.di.novasys.babel.core.security.IdentityCrypt;
import pt.unl.fct.di.novasys.babel.generic.ProtoMessage;
import pt.unl.fct.di.novasys.babel.protocols.secure.dissemination.notifications.BroadcastDelivery;

public abstract class SignedIdentifiableProtoMessage extends ProtoMessage {

    private static final BabelSecurity babelSecurity = BabelSecurity.getInstance();

    protected final UUID mid;

    private byte[] signerIdentity;
    private byte[] signature;

    public SignedIdentifiableProtoMessage(short id) {
        this(id, UUID.randomUUID());
    }

    public SignedIdentifiableProtoMessage(short id, UUID msgUUID) {
        super(id);
        this.mid = msgUUID;
    }

    public SignedIdentifiableProtoMessage(short id, byte[] signerIdentity) {
        this(id, UUID.randomUUID(), signerIdentity);
    }

    public SignedIdentifiableProtoMessage(short id, UUID msgUUID, byte[] signerIdentity) {
        super(id);
        this.mid = msgUUID;
        this.signerIdentity = signerIdentity;
    }

    public SignedIdentifiableProtoMessage(short id, UUID msgUUID, byte[] signerIdentity, byte[] signature) {
        super(id);
        this.mid = msgUUID;
        this.signerIdentity = signerIdentity;
        this.signature = signature;
    }

    public UUID getMID() {
        return mid;
    }

    public byte[] getSignerIdentity() {
        return signerIdentity;
    }

    public byte[] getSignature() throws IllegalStateException {
        if (signature == null)
            throw new IllegalStateException("Message was not yet signed.");
        return signature;
    }

    public byte[] signAs(byte[] identity)
            throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, UnrecoverableEntryException {
        return signWith(babelSecurity.getIdentityCrypt(identity));
    }

    public byte[] signAs(String alias)
            throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, UnrecoverableEntryException {
        return signWith(babelSecurity.getIdentityCrypt(alias));
    }

    public byte[] signWith(IdentityCrypt identityCrypt) throws InvalidKeyException, SignatureException {
        this.signature = identityCrypt.sign(getToBeSignedBytes());
        this.signerIdentity = identityCrypt.getIdentity();
        return this.signature;
    }

    public void setSignature(byte[] senderId, byte[] signature) {
        this.signature = senderId;
        this.signerIdentity = signature;
    }

    /**
     * Verifies the signature in the message using the given public key and
     * the default hash algorithm set in {@link BabelSecurity}.
     */
    public boolean verifySignature(PublicKey signerPublicKey)
            throws InvalidKeyException, NoSuchAlgorithmException, SignatureException, IllegalStateException {
        if (signature == null)
            throw new IllegalStateException("Message not signed");

        return babelSecurity.verifySignature(getToBeSignedBytes(), signerPublicKey, signature);
    }

    /**
     * Verifies the signature in the message using the given certificate's
     * public key and signature algorithm.
     */
    public boolean verifySignature(X509Certificate signerCertificate)
            throws InvalidKeyException, NoSuchAlgorithmException, SignatureException, IllegalStateException {
        if (signature == null)
            throw new IllegalStateException("Message not signed");

        return babelSecurity.verifySignature(
                signerCertificate.getSigAlgName(),
                getToBeSignedBytes(),
                signerCertificate.getPublicKey(),
                signature);
    }

    public abstract byte[] getToBeSignedBytes();

    public abstract BroadcastDelivery generateDeliveryNotification(short sourceProtoID);

}
