package pt.unl.fct.di.novasys.babel.protocols.storage.replies;

import com.google.gson.Gson;

import pt.unl.fct.di.novasys.babel.protocols.storage.replies.common.CommonExecuteReply;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.utils.CommonOperationStatus;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.utils.CommonOperationType;

/**
 * A reply of an operation parsed as a JSON string. The object representing the
 * result is passed as an argument in the construct and converted to a JSON
 * string by the GSON package. It stores information about the result of the
 * operation as well as the type of operation (e.g., READ, UPDATE, etc.) issued.
 */
public class ExecuteJSONReply extends CommonExecuteReply {
	public final static short REPLY_ID = 607;
	public final static Gson gson = new Gson();

	private String jsonString;

	/**
	 * Creates a execute JSON reply with the status of the operation, the type of
	 * operation issued, and an object representing the result, that is going to be
	 * converted to a JSON string. No information about the keySpace or collection
	 * is passed
	 * 
	 * @param status the status of the operation.
	 * @param type   the type of operation committed.
	 * @param result the result of the operation.
	 */
	public ExecuteJSONReply(CommonOperationStatus status, CommonOperationType type, Object result) {
		super(status, type, null, null, null, REPLY_ID);

		this.jsonString = gson.toJson(result);
	}

	/**
	 * Creates a execute JSON reply with the status of the operation, the type of
	 * operation issued, and an object representing the result, that is going to be
	 * converted to a JSON string.
	 * 
	 * @param status     the status of the operation.
	 * @param type       the type of operation committed.
	 * @param result     the result of the operation.
	 * @param keySpace   the keySpace where the operation was executed
	 * @param collection the collection where the operation was executed
	 * @param objectID   the identifier of the object
	 */
	public ExecuteJSONReply(CommonOperationStatus status, CommonOperationType type, String keySpace, String collection,
			String objectID, Object result) {
		super(status, type, keySpace, collection, objectID, REPLY_ID);

		this.jsonString = gson.toJson(result);
	}

	/**
	 * Creates a execute JSON reply with the status of the operation, the type of
	 * operation issued, and an object representing the result, that is going to be
	 * converted to a JSON string. The message should detail the status of the
	 * operation.
	 * 
	 * @param status     the status of the operation.
	 * @param type       the type of operation committed.
	 * @param keySpace   the keySpace where the operation was executed
	 * @param collection the collection where the operation was executed
	 * @param objectID   the identifier of the object
	 * @param result     the result of the operation.
	 * @param message    a message detailing the status.
	 * 
	 */
	public ExecuteJSONReply(CommonOperationStatus status, CommonOperationType type, String keySpace, String collection,
			String objectID, String message, Object result) {
		super(status, type, keySpace, collection, objectID, message, REPLY_ID);

		this.jsonString = gson.toJson(result);
	}

	/**
	 * Gets the result of the operation as a JSON string.
	 * 
	 * @return the result of the operation.
	 */
	public String getResult() {
		return jsonString;
	}
	/*
	 * 
	 * static class IteratorSerializer implements JsonSerializer<Iterator<?>> {
	 * 
	 * @Override public JsonElement serialize(Iterator<?> src,
	 * java.lang.reflect.Type typeOfSrc, JsonSerializationContext context) {
	 * LinkedList<Object> list = new LinkedList<>(); while (src.hasNext()) {
	 * list.add(src.next()); } return context.serialize(list); }
	 * 
	 * }
	 */
}
