package pt.unl.fct.di.novasys.babel.protocols.storage.replies;

import pt.unl.fct.di.novasys.babel.protocols.storage.replies.common.CommonExecuteReply;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.utils.CommonOperationStatus;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.utils.CommonOperationType;

/**
 * A reply of an operation that contains only the status of an operation. This
 * could be used in cases that only a status is expected from the storage layer
 * (i.e., OK or ERROR). It stores information about the status of the operation
 * as well as the type of operation committed.
 */
public class ExecuteStatusReply extends CommonExecuteReply {
	public final static short REPLY_ID = 608;

	/**
	 * Creates a execute status reply, with the type of operation issued on the
	 * request. Since a status is not passed, the class assumes the default OK. No
	 * information about the keySpace or collection is passed.
	 * 
	 * @param type the type of operation issued.
	 */
	public ExecuteStatusReply(CommonOperationType type) {
		super(CommonOperationStatus.OK, type, null, null, null, REPLY_ID);
	}

	/**
	 * Creates a execute status reply, with the type of operation issued on the
	 * request. Since a status is not passed, the class assumes the default OK. No
	 * information about the keySpace or collection is passed.
	 * 
	 * @param status the status of the operation.
	 * @param type   the type of operation issued.
	 */
	public ExecuteStatusReply(CommonOperationStatus status, CommonOperationType type) {
		super(status, type, null, null, null, REPLY_ID);
	}

	/**
	 * Creates a execute status reply, with the type of operation issued on the
	 * request. Since a status is not passed, the class assumes the default OK.
	 * 
	 * @param type       the type of operation issued.
	 * @param keySpace   the keySpace where the operation was executed
	 * @param collection the collection where the operation was executed
	 * @param objectID   the identifier of the object
	 */
	public ExecuteStatusReply(CommonOperationType type, String keySpace, String collection, String objectID) {
		super(CommonOperationStatus.OK, type, keySpace, collection, objectID, REPLY_ID);
	}

	/**
	 * Creates a execute status reply, with the type of operation issued on the
	 * request, as well as the status of the operation.
	 * 
	 * @param type       the type of operation issued.
	 * @param status     the status of the operation.
	 * @param keySpace   the keySpace where the operation was executed
	 * @param collection the collection where the operation was executed
	 * @param objectID   the identifier of the object
	 */
	public ExecuteStatusReply(CommonOperationStatus status, CommonOperationType type, String keySpace,
			String collection, String objectID) {
		super(status, type, keySpace, collection, objectID, REPLY_ID);
	}

	/**
	 * Creates a execute status reply, with the type of operation issued on the
	 * request, as well as the status of the operation. The message should contain
	 * details about the status passed.
	 * 
	 * @param type       the type of operation issued.
	 * @param status     the status of the operation.
	 * @param message    a message detailing the status.
	 * @param keySpace   the keySpace where the operation was executed
	 * @param collection the collection where the operation was executed
	 * @param objectID   the identifier of the object
	 * 
	 */
	public ExecuteStatusReply(CommonOperationStatus status, CommonOperationType type, String keySpace,
			String collection, String objectID, String message) {
		super(status, type, keySpace, collection, objectID, message, REPLY_ID);
	}

}
