package pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations;

import pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.common.CommonOperation;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.utils.CommonOperationType;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.types.ReplicatedStructuresOperations;
import pt.unl.fct.di.novasys.babel.protocols.storage.utils.types.ReplicatedStructuresOperations.ReplicatedDataTypes;

/**
 * An operation to performed on top of replicated data structure on the storage
 * layer. The operation contains information about the object identifier, the
 * operation to be executed on top of the replicated data structure (see more
 * information on ReplicatedStructuresOperations) as well as the value to be
 * applied on the operation.
 * 
 * This operation is meant to be used when adding, updating, reading or deleting
 * objects inside a previously created replicated data structure.
 */
public class ReplicatedOperation extends CommonOperation {
	private static final String SPLITTER = "#";

	private String objectID;
	private ReplicatedStructuresOperations operationToExecute;
	private Object value;

	/**
	 * Creates a replicated operation, with the operation type (e.g., CREATE, READ,
	 * etc .), the object identifier, the type of operation on top of the data
	 * structure (e.g., SET_ADD, MAP_PUT, etc.) and the value to be applied in the
	 * operation.
	 * 
	 * @param opType    the type of the operation
	 * @param objectID  the object identifier
	 * @param operation the type of operation on top of the data structure
	 * @param value     the value to be applied
	 */
	public ReplicatedOperation(CommonOperationType opType, String objectID, ReplicatedStructuresOperations operation,
			Object value) {
		super(opType);

		this.objectID = objectID;
		this.operationToExecute = operation;
		this.value = value;
	}

	/**
	 * Gets the object identifier
	 * 
	 * @return the object identifier
	 */
	public String getObjectID() {
		return objectID;
	}

	/**
	 * Get the type of the replicated data structure.
	 * 
	 * @return the type of the replicated data structure.
	 */
	public ReplicatedDataTypes getDataType() {
		String[] tokens = objectID.split(SPLITTER);
		return ReplicatedDataTypes.valueOf(tokens[1]);
	}

	/**
	 * Get the type operation being performed on top of the data structure.
	 * 
	 * @return the type operation being performed on top of the data structure.
	 */
	public ReplicatedStructuresOperations getStructureOperation() {
		return operationToExecute;
	}

	/**
	 * The value to be applied on the operation.
	 * 
	 * @return value to be applied on the operation.
	 */
	public Object getValue() {
		return value;
	}
}
