package pt.unl.fct.di.novasys.babel.protocols.storage.utils.operations.utils;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import io.netty.buffer.ByteBuf;
import pt.unl.fct.di.novasys.network.ISerializer;

/**
 * An auxiliary class to enable defining a set of conditions to be applied to a list of objects.
 * The class stores a Map of (String,List(String)) values, where each String represents the identifier of an object, and the 
 * List(String)being the list of conditions passed as a strings to be applied to the objects.
 */
public class ConditionsOnObjects {
	private Map<String, List<String>> conditions;

	/**
	 * Creates a list of conditions to be applied to a set of objects.
	 * @param conditions the conditions
	 */
	public ConditionsOnObjects(Map<String, List<String>> conditions) {
		this.conditions = new HashMap<>(conditions);
	}

	/**
	 * Gets the conditions to be applied to the objects.
	 * @return the conditions to be applied to the objects.
	 */
	public Map<String, List<String>> getConditions() {
		return conditions;
	}

	/**
	 * Serializer for the conditions on objects collection.
	 */
	public static ISerializer<ConditionsOnObjects> serializer = new ISerializer<ConditionsOnObjects>() {

		@Override
		public void serialize(ConditionsOnObjects o, ByteBuf out) throws IOException {
			Map<String, List<String>> conditions = o.getConditions();
			out.writeInt(conditions.keySet().size());
			for (Map.Entry<String, List<String>> entry : conditions.entrySet()) {
				encodeUTF8(entry.getKey(), out);
				out.writeInt(entry.getValue().size());

				List<String> conditionsObject = entry.getValue();
				for (String condition : conditionsObject) {
					encodeUTF8(condition, out);
				}
			}
		}

		@Override
		public ConditionsOnObjects deserialize(ByteBuf in) throws IOException {
			Map<String, List<String>> conditions = new HashMap<>();
			int numObjects = in.readInt();
			for (int i = 0; i < numObjects; i++) {
				String objectID = decodeUTF8(in);
				List<String> objectConditions = new LinkedList<String>();

				int numConditionsObject = in.readInt();
				for (int j = 0; j < numConditionsObject; j++) {
					objectConditions.add(decodeUTF8(in));
				}

				conditions.put(objectID, objectConditions);
			}

			return new ConditionsOnObjects(conditions);
		}
	};

	private static void encodeUTF8(String str, ByteBuf out) {
		byte[] stringBytes = str.getBytes(StandardCharsets.UTF_8);
		out.writeInt(stringBytes.length);
		out.writeBytes(stringBytes);
	}

	private static String decodeUTF8(ByteBuf in) {
		int length = in.readInt();
		byte[] stringBytes = new byte[length];
		in.readBytes(stringBytes);
		return new String(stringBytes, StandardCharsets.UTF_8);
	}
}
